#include <config.h>
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

#include <vmtrace.h>

#include <util.h>
#include <options.h>
#include <pagetable.h>
#include <physmem.h>
#include <simulate.h>
#include <stats.h>
#include <input.h>

static void sim_end_of_line();

/* refs per '.' printed */
const static uint dot_interval = 65536;
const static uint dots_per_line = 64;

void simulate() {
  uint length;
  vaddr_t vaddr;
  ref_kind_t type;
  pte_t *pte;
  fault_handler_t handler;
  uint count = 0;

  handler = opts.fault_handler->handler;
  
  printf("  ");
  while (input_next_ref(&vaddr, &type, &length)) {
    stats_reference(type);
    count++;
    if (opts.verbose && (count % dot_interval) == 0) {
      /* this process has no measurable effect on running time */
      printf(".");
      fflush(stdout); 
      if ((count % (dots_per_line * dot_interval)) == 0) {
	sim_end_of_line();
      }
    }
    
    pte = pagetable_lookup_vaddr(vaddr_to_vfn(vaddr), type);
    if (!pte->valid) {
      /* Fault */
      stats_miss(type);
      handler(pte, type);
    }
    
    pte->reference = 1;
    if (type == REF_KIND_STORE)
      pte->modified = TRUE;

    if (opts.limit && count >= opts.limit) {
      if (opts.verbose)
	printf("\nvmtrace: reached %d references\n", count);
      break;
    }

  }
}

void sim_end_of_line() {
  int curr, len;
  input_get_stats(&curr, &len);
  if (len != -1)
    printf(" %5.1f%%\n  ", 100*((double)curr)/((double)len));
}
